/// <summary>
///  Elemento grfico
///  @author Nacho Cabanes
///  @see Hardware Imagen
/// </summary>

/* --------------------------------------------------         
   Parte de FreddyTwo
 
   Versiones hasta la fecha:
   
   Num.   Fecha       Por / Cambios
   ---------------------------------------------------
   0.01  03-Feb-2012  Nacho Cabanes
                      Versin inicial: desglosado a partir de 
                        FreddyOne (Juego.cs)
 ---------------------------------------------------- */

public class ElemGrafico
{
    // Atributos

    protected bool visible;
    protected int x;
    protected int y;
    protected int xInicial;
    protected int yInicial;
    protected int ancho;
    protected int alto;
    protected int incrX;
    protected int incrY;
    
    // Si tiene solo una imagen
    protected Imagen imagen;

    // Si tiene una secuencia de imgenes
    protected Imagen[][] secuencia;
    protected byte fotogramaActual;
    protected byte direccion;
    protected bool contieneSecuencia = false;

    // Direcciones de movimiento
    protected byte direcciones = 4;
    public const byte ABAJO = 0;
    public const byte ARRIBA = 1;
    public const byte DERECHA = 2;
    public const byte IZQUIERDA = 3;

    // Operaciones

    /// Constructor: Carga la imagen que representara a este elemento grafico
    public ElemGrafico(string nombre)
    {
        imagen = new Imagen(nombre);
        visible = true;
        ancho = 32;
        alto = 32;
    }

    /// Constructor: Carga la imagen que representara a este elemento grafico
    public ElemGrafico(string nombre, int nuevoAncho, int nuevoAlto)
    {
        imagen = new Imagen(nombre);
        visible = true;
        ancho = nuevoAncho;
        alto = nuevoAlto;
    }

    // Constructor para una secuencia de imagenes
    public ElemGrafico(byte cantidadDirecciones, int nuevoAncho, int nuevoAlto)
    {
        contieneSecuencia = true;

        if (cantidadDirecciones > this.direcciones)
            this.direcciones = cantidadDirecciones;

        secuencia = new Imagen[direcciones][];
        visible = true;
        ancho = nuevoAncho;
        alto = nuevoAlto;
    }


    // Cargar una secuencia de imgenes
    public void CargarSecuencia(byte direcc, string[] nombres)
    {
        if (!contieneSecuencia)
            return;

        byte tamanyo = (byte)nombres.Length;
        secuencia[direcc] = new Imagen[tamanyo];
        for (byte i = 0; i < nombres.Length; i++)
            secuencia[direcc][i] = new Imagen(nombres[i]);
    }


    /// Mueve el elemento grafico a otra posicion
    public void CambiarDireccion(byte nuevaDir)
    {
        if (!contieneSecuencia)
            return;

        if (direccion != nuevaDir)
        {
            direccion = nuevaDir;
            fotogramaActual = 0;            
        }

    }


    /// Prepara el siguiente fotograma, para animar el movimiento de
    /// un personaje
    public void SiguienteFotograma()
    {
        if (!contieneSecuencia)
            return;

        if (fotogramaActual < secuencia[direccion].Length - 1)
            fotogramaActual++;
        else
            fotogramaActual = 0;
    }


    /// Dibuja el elemento grfico en sus coordenadas actuales  
    public  void DibujarOculta()
    {
      if (! visible) return;

      if (contieneSecuencia)
            secuencia[direccion][fotogramaActual].DibujarOculta(x, y);
      else 
          imagen.DibujarOculta(x, y);
    }

    /// Mueve un elemento grfico a otras coordenadas
    /// @param nuevaX Nueva posicin X (horizontal)
    /// @param nuevaY Nueva posicin Y (vertical)

    public  void MoverA(int nuevaX, int nuevaY)
    {
        x = nuevaX;
        y = nuevaY;
    }

    /// Mueve un elemento grfico a otras coordenadas y lo dibuja en ellas
    /// @param nuevaX Nueva posicin X (horizontal)
    /// @param nuevaY Nueva posicin Y (vertical)
    public  void DibujarOculta(int x, int y)
    {
        MoverA(x, y);
        DibujarOculta();
    }

    /// Comprueba si el elemento grfico choca con otro
    /// @param e Elemento grfico con el que comprobar colisiones
    /// @return true si sus rectngulos se solapan; false en caso contrario
    public  bool ColisionCon(ElemGrafico e)
    {
        // No se debe chocar con un elemento oculto
        if ((visible == false) || (e.visible == false))
            return false;
        // Ahora ya compruebo coordenadas
        if ((x + ancho > e.x)
              && (x < e.x + e.ancho)
              && (y + alto > e.y)
              && (y < e.y + e.alto))
            return true;
        else
            return false;
    }

    /// Comprueba si choca con un rectangulo
    public bool ColisionCon(int nx, int ny,
      int nxmax, int nymax)
    {
        // No se debe chocar con un elemento oculto      
        if (visible == false)
            return false;
        // Ahora ya compruebo coordenadas
        if ((nxmax > x)
            && (nymax > y)
            && (x + ancho > nx)
            && (y + alto > ny))
            return true;
        else
            return false;
    }



    /// Devuelve el valor de x
    public int GetX()
    {
        return x;
    }

    /// Devuelve el valor de y
    public int GetY()
    {
        return y;
    }

    /// Devuelve el valor de x
    public int GetXFin()
    {
        return x+ancho;
    }

    /// Devuelve el valor de y
    public int GetYFin()
    {
        return y+alto;
    }

    /// Devuelve la velocidad horizontal
    public int GetVelocX()
    {
        return incrX;
    }

    /// Devuelve la velocidad vertical
    public int GetVelocY()
    {
        return incrY;
    }

    /// Cambia el ancho y el alto
    public void SetAnchoAlto(short an, short al)
    {
        alto = al;
        ancho = an;
    }

    /// Devuelven ancho y alto
    public short GetAncho()
    {
        return (short) ancho;
    }

    public short GetAlto()
    {
        return (short) alto;
    }

    /// Devuelve si est visible
    public bool GetVisible()
    {
        return visible;
    }

    /// Cambia si est visible o no )
    public void SetVisible(bool a)
    {
        visible = a;
    }


} /* Fin de la clase ElemGrafico */
