/// <summary>
///  Nivel: el nivel actual de juego
///  @author Nacho Cabanes
///  @see Menu Juego
/// </summary>

/* --------------------------------------------------         
   Parte de FreddyTwo
 
   Versiones hasta la fecha:
   
   Num.   Fecha       Por / Cambios
   ---------------------------------------------------
   0.01  03-Feb-2012  Nacho Cabanes
                      Versin inicial: desglosado a partir de 
                        FreddyOne (Juego.cs)
   0.02  04-Feb-2012  Cristina Giner Olcina y Jorge Bernabeu Mira
                      Varios niveles consecutivos
 ---------------------------------------------------- */

using System;  // Para Random
using System.IO;  // Para fichero de niveles

public class Nivel
{
    // Atributos
    Random generador;

    //private int tiempoMax;
    private int premiosRestantes;
    private int premiosVisibles, enemigosVisibles, obstaculosVisibles;

    byte anchoFondo = 24;
    byte altoFondo = 17;
    short margenXFondo = 10;
    byte margenYFondo = 30;
    byte anchoCasillaFondo = 32;
    byte altoCasillaFondo = 32;

    int numPremios, numEnemigos, numObstaculos;

    Obstaculo[] obstaculos;
    Enemigo[] enemigos;
    Premio[] premios;
    ElemGrafico[] fondos;
    int elementosFondo;
    int nivel;

    int xPersonaje, yPersonaje;


    public char[, ,] fondo;


    public Nivel()
    {
        leerDesdeFichero();
        nivel = 0;
        RellenarArrayDeFondo();
    }

    private void leerDesdeFichero()
    {
        StreamReader ficheroMapas = File.OpenText("niveles.dat");
        string linea;

        linea = ficheroMapas.ReadLine();
        int numeroNiveles = Convert.ToInt32(linea);

        fondo = new char[numeroNiveles, altoFondo, anchoFondo];

        for (int nivelActual = 0; nivelActual < numeroNiveles; nivelActual++)
        {
            string separador = ficheroMapas.ReadLine();

            for (int lineaActual = 0; lineaActual < altoFondo; lineaActual++)
            {
                linea = ficheroMapas.ReadLine();
                for (int columnaActual = 0; columnaActual < anchoFondo; columnaActual++)
                    fondo [nivelActual, lineaActual, columnaActual] =  linea[columnaActual];
                // Extraigo las columnas
            }
        }

        ficheroMapas.Close();
    }


    //funcion auxiliar
    public void RellenarArrayDeFondo()
    {
        
        elementosFondo = 0;
        numPremios = 0;
        numEnemigos = 0;
        numObstaculos = 0;
        for (int fila = 0; fila < altoFondo; fila++)  // Fondo
            for (int col = 0; col < anchoFondo; col++)
            {
                // Elementos del fondo: 1 al 9
                if ((fondo[nivel, fila, col] >= '1') && (fondo[nivel, fila, col] <= '9'))
                    elementosFondo++;
                // Premios: X
                if (fondo[nivel, fila, col] == 'X')
                    numPremios++;
                // Enemigos: E
                if (fondo[nivel, fila, col] == 'E')
                    numEnemigos++;
                // Obstculos: O
                if (fondo[nivel, fila, col] == 'O')
                    numObstaculos++;
                // Posicion del personaje
                if (fondo[nivel, fila, col] == 'P')
                {
                    xPersonaje = margenXFondo + col * anchoCasillaFondo;
                    yPersonaje = margenYFondo + fila * altoCasillaFondo;
                }
            }
        premiosRestantes = numPremios;
        // Y reservo espacio para el array que los contiene
        fondos = new ElemGrafico[elementosFondo];
        obstaculos = new Obstaculo[numObstaculos];
        enemigos = new Enemigo[numEnemigos];
        premios = new Premio[numPremios];

        int posicFondo = 0, contEnemigos = 0, contObstaculos = 0, contPremios = 0;
        for (int fila = 0; fila < altoFondo; fila++)  // Fondo
            for (int col = 0; col < anchoFondo; col++)
                switch (fondo[nivel, fila, col])
                {
                    case '1':
                        fondos[posicFondo] = new ElemGrafico("pared.png", anchoCasillaFondo, altoCasillaFondo);
                        fondos[posicFondo].MoverA(margenXFondo + col * anchoCasillaFondo,
                            margenYFondo + fila * altoCasillaFondo);
                        fondos[posicFondo].SetVisible(true);
                        posicFondo++;
                        break;
                    case '2':
                        fondos[posicFondo] = new ElemGrafico("pared2.png", anchoCasillaFondo, altoCasillaFondo);
                        fondos[posicFondo].MoverA(margenXFondo + col * anchoCasillaFondo,
                            margenYFondo + fila * altoCasillaFondo);
                        fondos[posicFondo].SetVisible(true);
                        posicFondo++;
                        break;
                    case '3':
                        fondos[posicFondo] = new ElemGrafico("pared3.png", anchoCasillaFondo, altoCasillaFondo);
                        fondos[posicFondo].MoverA(margenXFondo + col * anchoCasillaFondo,
                            margenYFondo + fila * altoCasillaFondo);
                        fondos[posicFondo].SetVisible(true);
                        posicFondo++;
                        break;
                    case 'X':
                        premios[contPremios] = new Premio();
                        premios[contPremios].MoverA(margenXFondo + col * anchoCasillaFondo,
                            margenYFondo + fila * altoCasillaFondo);
                        premios[contPremios].SetVisible(true);
                        contPremios++;
                        break;
                    case 'O':
                        obstaculos[contObstaculos] = new Obstaculo();
                        obstaculos[contObstaculos].MoverA(margenXFondo + col * anchoCasillaFondo,
                            margenYFondo + fila * altoCasillaFondo);
                        obstaculos[contObstaculos].SetVisible(true);
                        contObstaculos++;
                        break;
                    case 'E':
                        enemigos[contEnemigos] = new Enemigo();
                        enemigos[contEnemigos].MoverA(margenXFondo + col * anchoCasillaFondo,
                            margenYFondo + fila * altoCasillaFondo);
                        enemigos[contEnemigos].SetVisible(true);
                        contEnemigos++;
                        break;
                }
    }


    /// Deja todo como es necesario para una nueva partida
    public void Reiniciar()
    {
        nivel = 0;
        RellenarArrayDeFondo();
    }


    public void DibujarOculta()
    {
        for (int i = 0; i < elementosFondo; i++)  // Imgenes del fondo
            fondos[i].DibujarOculta();

        for (int i = 0; i < numObstaculos; i++)  // Obstculos
            obstaculos[i].DibujarOculta();

        for (int i = 0; i < numEnemigos; i++)  // Enemigos
            enemigos[i].DibujarOculta();

        for (int i = 0; i < numPremios; i++)  // Premios
            premios[i].DibujarOculta();

    }

    // Devuelve si es posible mover a ciertas coordenadas
    // (no hay obstculos)
    public bool EsPosibleMover(int x, int y, int xFin, int yFin)
    {
        // Veo si choca con algn elemento del fondo
        for (int i = 0; i < elementosFondo; i++)
            if (fondos[i].ColisionCon(x,y,xFin,yFin))
                return false;

        // Si no ha chocado con ninguno, es posible moverse a esa posicin
        return true;
    }

    public int ObtenerPuntosPosicion(int x, int y, int xFin, int yFin)
    {
        // Veo si recoge algn premio
        for (int i = 0; i < numPremios; i++)
            if (premios[i].ColisionCon(x, y, xFin, yFin))
            {
                premios[i].SetVisible(false);
                premiosRestantes--;
                if (premiosRestantes == 0)
                    Avanzar();
                return 10;
            }

        // O si toca algn obstculo
        for (int i = 0; i < numObstaculos; i++)
            if (obstaculos[i].ColisionCon(x, y, xFin, yFin))
                return -1;

        // O si choca con algn enemigo
        for (int i = 0; i < numEnemigos; i++)
            if (enemigos[i].ColisionCon(x, y, xFin, yFin))
                return -1;

        // Si llego hasta aqui, es que no hay puntos que devolver
        return 0;

    }



    public int GetNumEnemigos()
    {
        return numEnemigos;
    }


    public Enemigo GetEnemigo(int num)
    {
        return enemigos[num];
    }


    public void Avanzar()
    {
        // Borro la pantalla y aviso del nuevo nivel
        Hardware.Pausa(200);
        Hardware.BorrarPantallaOculta();
        //Hardware.EscribirTextoOculta("Nivel: " + Convert.ToString(premiosVisibles + 1),
        //         350, 300, /* Coordenadas */ 255, 255, 255, /* Colores */ tipoDeLetra);
        Hardware.VisualizarOculta();
        Hardware.Pausa(1000);

        // Cambio los "ladrillos" del fondo
        if (nivel < 2)
            nivel++;
        else
            nivel = 0;

        RellenarArrayDeFondo();
    }

} /* Fin de la clase Nivel */

