/// <summary>
///  Partida: una sesin de juego
///  @author Nacho Cabanes
///  @see Menu Juego
/// </summary>

/* --------------------------------------------------         
   Parte de FreddyTwo
 
   Versiones hasta la fecha:
   
   Num.   Fecha       Por / Cambios
   ---------------------------------------------------
   0.01  03-Feb-2012  Nacho Cabanes
                      Versin inicial: desglosado a partir de 
                        FreddyOne (Juego.cs)
 ---------------------------------------------------- */

using System;

public class Partida
{
    // Attributes

    // private int tiempoJuego;
    private int puntos;
    private int vidas;
    private Nivel miNivel;
    private Personaje miPersonaje;
    private Marcador miMarcador;

    bool partidaTerminada;

    // Operaciones

    /// Constructor
    public Partida()
    {
        miMarcador = new Marcador();
        miPersonaje = new Personaje(this);
        miNivel = new Nivel();
    }

    /// Comienza la partida
    public  void Comenzar()
    {
        partidaTerminada = false;
        puntos = 0;
        miMarcador.SetPuntuacion(puntos);
        vidas = 3;
        miMarcador.SetVidas(vidas);
        miNivel.Reiniciar();

        // ------ Bucle de juego ------
        while (!partidaTerminada)
        {
            Dibujar();
            ComprobarTeclas();
            MoverElementos();
            ComprobarColisiones();
            PausaFotograma();
        } // Fin del bucle de juego
    }


    public void Dibujar()
    {
        // -- Dibujar --
        Hardware.BorrarPantallaOculta(0, 0, 0);

        // Marcador
        miMarcador.DibujarOculta();

        // Elementos del nivel: fondo, premios y enemigos
        miNivel.DibujarOculta();

        // Personaje
        miPersonaje.DibujarOculta();

        // Finalmente, muestro en pantalla
        Hardware.VisualizarOculta();
    }


    public void ComprobarTeclas()
    {
        // -- Leer teclas y calcular nueva posicin --
        if (Hardware.TeclaPulsada(Hardware.TECLA_ESC))
            partidaTerminada = true;

        if (((Hardware.TeclaPulsada(Hardware.TECLA_DER) && Hardware.TeclaPulsada(Hardware.TECLA_ESP)))
            || (Hardware.JoystickDerecha() && Hardware.JoystickPulsado(0)))
            miPersonaje.SaltarDerecha();

        else if (((Hardware.TeclaPulsada(Hardware.TECLA_IZQ) && Hardware.TeclaPulsada(Hardware.TECLA_ESP)))
            || (Hardware.JoystickIzquierda() && Hardware.JoystickPulsado(0)))
            miPersonaje.SaltarIzquierda();

        else if ((Hardware.TeclaPulsada(Hardware.TECLA_ARR) || Hardware.JoystickArriba())
            && miNivel.EsPosibleMover(miPersonaje.GetX(), miPersonaje.GetY() - miPersonaje.GetVelocY(),
                    miPersonaje.GetXFin(), miPersonaje.GetYFin() - miPersonaje.GetVelocY()))
            miPersonaje.MoverArriba();

        else if ((Hardware.TeclaPulsada(Hardware.TECLA_ABA) || Hardware.JoystickAbajo())
            && miNivel.EsPosibleMover(miPersonaje.GetX(), miPersonaje.GetY() + miPersonaje.GetVelocY(),
                   miPersonaje.GetXFin(), miPersonaje.GetYFin() + miPersonaje.GetVelocY()))
            miPersonaje.MoverAbajo();

        else if ((Hardware.TeclaPulsada(Hardware.TECLA_DER) || Hardware.JoystickDerecha())
            && miNivel.EsPosibleMover(miPersonaje.GetX() + miPersonaje.GetVelocX(), miPersonaje.GetY(),
                    miPersonaje.GetXFin() + miPersonaje.GetVelocX(), miPersonaje.GetYFin()))
            miPersonaje.MoverDerecha();

        else if ((Hardware.TeclaPulsada(Hardware.TECLA_IZQ) || Hardware.JoystickIzquierda())
            && miNivel.EsPosibleMover(miPersonaje.GetX() - miPersonaje.GetVelocX(), miPersonaje.GetY(),
                   miPersonaje.GetXFin() - miPersonaje.GetVelocX(), miPersonaje.GetYFin()))
            miPersonaje.MoverIquierda();

        else if (Hardware.TeclaPulsada(Hardware.TECLA_ESP) || Hardware.JoystickPulsado(0) )
            miPersonaje.Saltar();

    }


    public void MoverElementos()
    {
        // -- Mover enemigos, entorno --
        for (int i = 0; i < miNivel.GetNumEnemigos(); i++)
            miNivel.GetEnemigo(i).Mover();

        miPersonaje.Mover();
    }


    public void ComprobarColisiones()
    {
        // -- Colisiones, perder vidas, etc --

        // Primero pregunto al nivel actual qu pasa en esta posicn
        int puntosMovimiento = miNivel.ObtenerPuntosPosicion(miPersonaje.GetX(),
            miPersonaje.GetY(), miPersonaje.GetXFin(), miPersonaje.GetYFin());
        
        // Si ha chocado con algo, hay que perder una vida
        if ( puntosMovimiento == -1 )
        {
            vidas--;
            if (vidas == 0)
                partidaTerminada = true;
            miMarcador.SetVidas(vidas);
            miPersonaje.Reiniciar();
        }
        // Si no ha chocado, sumo los puntos obtenidos
        else
        {
            puntos += puntosMovimiento;
            miMarcador.IncrPuntuacion(puntosMovimiento);
        }
    }


    public void PausaFotograma()
    {
        // -- Pausa hasta el siguiente "fotograma" del juego --
        Hardware.Pausa(20);
    }

    // Devuelve si es posible mover a ciertas coordenadas
    // (no hay obstculos)
    // (Delega en el "nivel" para que lo calcule)
    public bool EsPosibleMover(int x, int y, int xFin, int yFin)
    {
        return miNivel.EsPosibleMover(x,y,xFin,yFin);
    }

} /*Fin de la clase Partida */
