/// <summary>
///  Personaje: el elemento que mueve el usuario
///  @author Nacho Cabanes
///  @see Hardware Juego ElemGrafico
/// </summary>

/* --------------------------------------------------         
   Parte de FreddyTwo
 
   Versiones hasta la fecha:
   
   Num.   Fecha       Por / Cambios
   ---------------------------------------------------
   0.01  03-Feb-2012  Nacho Cabanes
                      Versin inicial: desglosado a partir de 
                        FreddyOne (Juego.cs)
 ---------------------------------------------------- */

public class Personaje : ElemGrafico
{
    // Atributos
    bool saltando;
    int[] pasosSalto =
    { -12, -12, -10, -10, -8, -8, -6, -6, -4, -2, -1, -1, 0,
         0, 1, 1, 2, 4, 6, 6, 8, 8, 10, 10, 12, 12};
    int pasoSaltoActual;
    int incrXsalto;
    //private int vidas;

    // Asociaciones

    private Partida miPartida;

    // Operaciones

    public Personaje(Partida p)
        : base(4, 32, 23)  // Elemento grfico con (supuestamente) 4 direcciones
    {
        miPartida = p;
        xInicial = 400;
        yInicial = 300;
        x = xInicial;
        y = yInicial;
        visible = true;
        incrX = 10;
        incrY = 10;
        CargarSecuencia(DERECHA,new string[] { "personajed1.png", "personajed2.png" });
        CargarSecuencia(IZQUIERDA, new string[] { "personajei1.png", "personajei2.png" });
        CambiarDireccion(DERECHA);
        saltando = false;
        pasoSaltoActual = 0;
        incrXsalto = 0;
    }

    /// Devuelve el personaje a su posicin inicial
    public void Reiniciar()
    {
        x = xInicial;
        y = yInicial;
    }

    

    /// Hace que el personaje "lance" un disparo
    public  void Disparar()
    {
        // TODO: Pendiente de completar
    }

    /// Mueve el personaje hacia arriba (si es posible en el nivel actual)
    /// Las comprobaciones deben hacerse "fuera", antes de llamar a MoverDerecha
    public void MoverArriba()
    {
        if (saltando)
            return;

        SiguienteFotograma();
        y -= incrY;
    }

    /// Mueve el personaje hacia abajo (si es posible en el nivel actual)
    /// Las comprobaciones deben hacerse "fuera", antes de llamar a MoverIzquierda
    public void MoverAbajo()
    {
        if (saltando)
            return;

        SiguienteFotograma();
        y += incrY;
    }

    /// Mueve el personaje a la derecha (si es posible en el nivel actual)
    /// Las comprobaciones deben hacerse "fuera", antes de llamar a MoverDerecha
    public  void MoverDerecha()
    {
        if (saltando)
            return;

        CambiarDireccion(DERECHA);
        SiguienteFotograma();
        x += incrX;
    }

    /// Mueve el personaje a la izquierda (si es posible en el nivel actual)
    /// Las comprobaciones deben hacerse "fuera", antes de llamar a MoverIzquierda
    public  void MoverIquierda()
    {
        if (saltando)
            return;

        CambiarDireccion(IZQUIERDA);
        SiguienteFotograma();
        x -= incrX;
    }

    /// Hace que el personaje "baje" si no tiene suelo debajo
    public  void Caer()
    {
        y += incrY;
    }

    /// Hace que el personaje comience a saltar a la izquierda
    public void SaltarIzquierda()
    {
        saltando = true;
        incrXsalto = -4;
    }

    /// Hace que el personaje comience a saltar a la derecha
    public void SaltarDerecha()
    {
        saltando = true;
        incrXsalto = 4;
    }

    /// Hace que el personaje comience a saltar en vertical
    public void Saltar()
    {
        saltando = true;
        incrXsalto = 0;
    }


    /// Hace que el personaje se siga moviendo (si debe hacerlo)
    /// De momento, slo llama a "Caer"
    public void Mover()
    {
        if (saltando)
        {
            if (miPartida.EsPosibleMover(
                    x + incrXsalto, 
                    y + pasosSalto[pasoSaltoActual],
                    x + incrXsalto + ancho, 
                    y + alto + pasosSalto[pasoSaltoActual]))
            {
                // Siguiente incremento del salto
                y += pasosSalto[pasoSaltoActual];
                x += incrXsalto;
                // Si no ha terminado la secuencia
                if (pasoSaltoActual < pasosSalto.Length-1)
                    pasoSaltoActual++; // preparo el siguiente
                // Y si ha terminado, reinicio
                else
                {
                    saltando = false;
                    pasoSaltoActual = 0;
                }
            }
            else // Si no se puede mover, fin del salto
            {
                saltando = false;
                pasoSaltoActual = 0;
            }
        }
        else
            if (miPartida.EsPosibleMover(x, y + incrY,
                    x + ancho, y + alto + incrY))
                Caer();
    }


} /* Fin de la clase Personaje */
